# /mods/CampaignBranch/ExtractMPBlueprints.lua
#
# Run this script with ExtractMPBlueprints.bat

local datadir = '../../'
local outfile = 'all_units.bp'
local current_id

local all_blueprints = {}


dofile(datadir..'lua/system/repr.lua')
dofile(datadir..'lua/system/utils.lua')


function ExtractFields(bp)

    # Force sub-tables to exist so we don't get an error trying to access them
    bp.Defense = bp.Defense or {}
    bp.Economy = bp.Economy or {}
    bp.Physics = bp.Physics or {}

    local r = {
        Merge = true,   # flag to merge with original BP rather than replacing it
        BlueprintId = bp.BlueprintId,
        Categories = bp.Categories,
        Air = bp.Air,
        Defense = {
            ArmorType = bp.Defense.ArmorType,
            Health = bp.Defense.Health,
            MaxHealth = bp.Defense.MaxHealth,
            RegenRate = bp.Defense.RegenRate,
        },
        Economy = {
            BuildCostEnergy = bp.Economy.BuildCostEnergy,
            BuildCostMass = bp.Economy.BuildCostMass,
            BuildTime = bp.Economy.BuildTime,
            BuildRate = bp.Economy.BuildRate,
            BuildableCategory = bp.Economy.BuildableCategory,
            MaxEnergyUse = bp.Economy.MaxEnergyUse,
            MaxMassUse = bp.Economy.MaxMassUse,
            StorageEnergy = bp.Economy.StorageEnergy,
            StorageMass = bp.Economy.StorageMass,
            TeleportEnergyMod = bp.Economy.TeleportEnergyMod,
            TeleportMassMod = bp.Economy.TeleportMassMod,
            TeleportTimeMod = bp.Economy.TeleportTimeMod,
            MaintenanceConsumptionPerSecondEnergy = bp.Economy.MaintenanceConsumptionPerSecondEnergy,
            MaintenanceConsumptionPerSecondMass = bp.Economy.MaintenanceConsumptionPerSecondMass,
            ProductionPerSecondMass = bp.Economy.ProductionPerSecondMass,
            ProductionPerSecondEnergy = bp.Economy.ProductionPerSecondEnergy,
        },
        Footprint = bp.Footprint,
        Intel = bp.Intel,
        Physics = {
            BankingSlope = bp.Physics.BankingSlope,
            MaxSteerForce = bp.Physics.MaxSteerForce,
            FuelRechargeRate = bp.Physics.FuelRechargeRate,
            FuelUseTime = bp.Physics.FuelUseTime,
            MaxSpeed = bp.Physics.MaxSpeed,
            MaxBrake = bp.Physics.MaxBrake,
            MaxSpeedReverse = bp.Physics.MaxSpeedReverse,
            MinSpeedPercent = bp.Physics.MinSpeedPercent,
            TurnRadius = bp.Physics.TurnRadius,
            TurnRate = bp.Physics.TurnRate,
            BuildOnLayerCaps = bp.Physics.BuildOnLayerCaps,
            SkirtOffsetX = bp.Physics.SkirtOffsetX,
            SkirtOffsetZ = bp.Physics.SkirtOffsetZ,
            SkirtSizeX = bp.Physics.SkirtSizeX,
            SkirtSizeZ = bp.Physics.SkirtSizeZ,
            MaxAcceleration = bp.Physics.MaxAcceleration,
        },
    }
    if bp.Weapon then
        r.Weapon = {}
        for i,wbp in ipairs(bp.Weapon) do
            r.Weapon[i] = {
                BallisticArc = wbp.BallisticArc,
                Damage = wbp.Damage,
                DoTPulses = wbp.DoTPulses,
                DoTTime = wbp.DoTTime,
                FireTargetLayerCapsTable = wbp.FireTargetLayerCapsTable,
                FiringRandomness = wbp.FiringRandomness,
                EnergyDrainPerSecond = wbp.EnergyDrainPerSecond,
                EnergyRequired = wbp.EnergyRequired,
                MaxRadius = wbp.MaxRadius,
                MinRadius = wbp.MinRadius,
                MuzzleVelocity = wbp.MuzzleVelocity,
                RateOfFire = wbp.RateOfFire,
                RackSalvoReloadTime = wbp.RackSalvoReloadTime,
                RackSalvoChargeTime = wbp.RackSalvoChargeTime,
                TrackingRadius = wbp.TrackingRadius,
            }
        end
    end
    return r
end


#===================================================================
# Define constructors that get called from the Blueprint files
#-------------------------------------------------------------------
function Sound()
end

function RPCSound()
end

function UnitBlueprint(bp)
    bp.BlueprintId = current_id
    all_blueprints[current_id] = ExtractFields(bp)
end


#===================================================================
# Generate the partial blueprint file
#-------------------------------------------------------------------
function LoadAllBlueprints()
    for i,d in io.dir(datadir .. 'units/*.*') do
        current_id = string.lower(d)

        local filename = string.format('%sunits/%s/%s_unit.bp', datadir, d, d)
        local ok,msg = pcall(dofile, filename)
        if not ok and not string.find(msg,"cannot read.*: No such file or directory") then
            error(msg)
        end
    end
end


function WriteAllBlueprints()
    local n = 0
    local file = io.open(outfile, "w")
    file:write("# Generated by ExtractMPBlueprints.lua (", os.date(), " ", os.getenv "USERNAME", ")\n")
    for i,id in table.keys(all_blueprints) do
        local bp = all_blueprints[id]
        file:write('UnitBlueprint ', repr(bp), '\n')
        n = n + 1
    end
    file:close()

    print('Wrote '..n..' blueprints to '..outfile)
end


LoadAllBlueprints()
WriteAllBlueprints()
